<?php

namespace Drupal\layout_builder_enhancements\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\layout_builder_enhancements\ViewsServiceInterface;
use Drupal\layout_builder_enhancements\Plugin\Block\ViewBlock;
use Drupal\layout_builder\LayoutBuilderEvents;
use Drupal\layout_builder\Event\SectionComponentBuildRenderArrayEvent;

/**
 * EventSubscriber for layout render event.
 */
class ViewEventSubscriber implements EventSubscriberInterface {

  protected $service;

  /**
   * Constructor.
   */
  public function __construct(ViewsServiceInterface $viewService) {
    $this->service = $viewService;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      LayoutBuilderEvents::SECTION_COMPONENT_BUILD_RENDER_ARRAY => [
        'onBuildRender',
        100,
      ],
    ];
  }

  /**
   * Event render function.
   */
  public function onBuildRender(SectionComponentBuildRenderArrayEvent $event) {
    $block = $event->getPlugin();
    if (!$block instanceof ViewBlock) {
      return;
    }
    $build = $event->getBuild();

    $view = $block->getView();
    $offset = $this->service->getOffset($view->id(), $block->getDisplayId(), $block->getDerivativeId());

    $block->setOffset($offset);
    $block->inPreview($event->inPreview());

    $build['content'] = $block->build();

    $this->service->setOffset($view->id(), $block->getDisplayId(), $block->getDerivativeId(), $offset + $block->getResultCount());

    $event->setBuild($build);
  }

}
