<?php

namespace Drupal\layout_builder_enhancements;
use Symfony\Component\Process\Exception\LogicException;
use Drupal\views\Views;
use Drupal\views\ViewExecutable;
use Drupal\views\Entity\View;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Class ViewsService.
 */
class ViewsService implements ViewsServiceInterface {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;
  /**
   * Constructs a new ViewsService object.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  public function getView($view_id, $display_id, $cache_id) : ViewExecutable {
    return $this->getViewFromCache($cache_id, $view_id, $display_id);
  }

  protected function getViewFromCache($cache_id, $view_id, $display_id, $offset = 0): ViewExecutable {
    $cache_key = implode(':', [$cache_id, $view_id, $display_id]);
    $cache = &drupal_static(__FUNCTION__);
    if (!$cache) {
      $cache = [];
    }
    if (isset($cache[$cache_key])) {
      return $cache[$cache_key];
    }
    $view = Views::getView($view_id);

    if (!$view || !$this->isApplicable($view)) {
      throw new LogicException('This view do not exists anymore');
    }
    $view->setDisplay($display_id);
    $view->execute();
    $cache[$cache_key] = $view;

    return $cache[$cache_key];
  }

  protected function isApplicable(ViewExecutable $view): bool {
    return true;
  }

  public function setOffset($view_id, $display_id, $cache_id, $offset) : void {
    $cache_key = implode(':', [$cache_id, $view_id, $display_id]);
    $cache = &drupal_static('layout_builder_enhancements_offsets');
    if (!isset($cache[$cache_key])) {
      $cache[$cache_key] = 0;
    }
    $cache[$cache_key] = $offset;
  }

  public function getOffset($view_id, $display_id, $cache_id) : int {
    $cache_key = implode(':', [$cache_id, $view_id, $display_id]);
    $cache = &drupal_static('layout_builder_enhancements_offsets');
    if (!isset($cache[$cache_key])) {
      $cache[$cache_key] = 0;
    }
    return $cache[$cache_key];
  }

}
